/****************************************************************************
** Meta object code from reading C++ file 'kompare_shell.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/kompare-17.04.3/kompare_shell.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kompare_shell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KompareShell_t {
    QByteArrayData data[24];
    char stringdata0[365];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareShell_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareShell_t qt_meta_stringdata_KompareShell = {
    {
QT_MOC_LITERAL(0, 0, 12), // "KompareShell"
QT_MOC_LITERAL(1, 13, 19), // "slotUpdateStatusBar"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 10), // "modelIndex"
QT_MOC_LITERAL(4, 45, 15), // "differenceIndex"
QT_MOC_LITERAL(5, 61, 10), // "modelCount"
QT_MOC_LITERAL(6, 72, 15), // "differenceCount"
QT_MOC_LITERAL(7, 88, 12), // "appliedCount"
QT_MOC_LITERAL(8, 101, 8), // "viewPart"
QT_MOC_LITERAL(9, 110, 17), // "KompareInterface*"
QT_MOC_LITERAL(10, 128, 20), // "slotSetStatusBarText"
QT_MOC_LITERAL(11, 149, 4), // "text"
QT_MOC_LITERAL(12, 154, 12), // "slotFileOpen"
QT_MOC_LITERAL(13, 167, 20), // "slotFileCompareFiles"
QT_MOC_LITERAL(14, 188, 23), // "slotFileBlendURLAndDiff"
QT_MOC_LITERAL(15, 212, 16), // "slotShowTextView"
QT_MOC_LITERAL(16, 229, 13), // "slotFileClose"
QT_MOC_LITERAL(17, 243, 20), // "optionsConfigureKeys"
QT_MOC_LITERAL(18, 264, 24), // "optionsConfigureToolbars"
QT_MOC_LITERAL(19, 289, 17), // "slotSetDiffString"
QT_MOC_LITERAL(20, 307, 10), // "diffString"
QT_MOC_LITERAL(21, 318, 16), // "newToolbarConfig"
QT_MOC_LITERAL(22, 335, 21), // "slotVisibilityChanged"
QT_MOC_LITERAL(23, 357, 7) // "visible"

    },
    "KompareShell\0slotUpdateStatusBar\0\0"
    "modelIndex\0differenceIndex\0modelCount\0"
    "differenceCount\0appliedCount\0viewPart\0"
    "KompareInterface*\0slotSetStatusBarText\0"
    "text\0slotFileOpen\0slotFileCompareFiles\0"
    "slotFileBlendURLAndDiff\0slotShowTextView\0"
    "slotFileClose\0optionsConfigureKeys\0"
    "optionsConfigureToolbars\0slotSetDiffString\0"
    "diffString\0newToolbarConfig\0"
    "slotVisibilityChanged\0visible"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareShell[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    5,   79,    2, 0x0a /* Public */,
       8,    0,   90,    2, 0x0a /* Public */,
      10,    1,   91,    2, 0x08 /* Private */,
      12,    0,   94,    2, 0x08 /* Private */,
      13,    0,   95,    2, 0x08 /* Private */,
      14,    0,   96,    2, 0x08 /* Private */,
      15,    0,   97,    2, 0x08 /* Private */,
      16,    0,   98,    2, 0x08 /* Private */,
      17,    0,   99,    2, 0x08 /* Private */,
      18,    0,  100,    2, 0x08 /* Private */,
      19,    1,  101,    2, 0x08 /* Private */,
      21,    0,  104,    2, 0x08 /* Private */,
      22,    1,  105,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    4,    5,    6,    7,
    0x80000000 | 9,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   23,

       0        // eod
};

void KompareShell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KompareShell *_t = static_cast<KompareShell *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->slotUpdateStatusBar((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 1: { KompareInterface* _r = _t->viewPart();
            if (_a[0]) *reinterpret_cast< KompareInterface**>(_a[0]) = _r; }  break;
        case 2: _t->slotSetStatusBarText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->slotFileOpen(); break;
        case 4: _t->slotFileCompareFiles(); break;
        case 5: _t->slotFileBlendURLAndDiff(); break;
        case 6: _t->slotShowTextView(); break;
        case 7: _t->slotFileClose(); break;
        case 8: _t->optionsConfigureKeys(); break;
        case 9: _t->optionsConfigureToolbars(); break;
        case 10: _t->slotSetDiffString((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->newToolbarConfig(); break;
        case 12: _t->slotVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject KompareShell::staticMetaObject = {
    { &KParts::MainWindow::staticMetaObject, qt_meta_stringdata_KompareShell.data,
      qt_meta_data_KompareShell,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareShell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareShell::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareShell.stringdata0))
        return static_cast<void*>(const_cast< KompareShell*>(this));
    return KParts::MainWindow::qt_metacast(_clname);
}

int KompareShell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::MainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
