/****************************************************************************
** Meta object code from reading C++ file 'komparenavtreepart.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kompare-17.04.3/komparenavtreepart/komparenavtreepart.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'komparenavtreepart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KompareNavTreePart_t {
    QByteArrayData data[24];
    char stringdata0[420];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareNavTreePart_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareNavTreePart_t qt_meta_stringdata_KompareNavTreePart = {
    {
QT_MOC_LITERAL(0, 0, 18), // "KompareNavTreePart"
QT_MOC_LITERAL(1, 19, 16), // "selectionChanged"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 23), // "const Diff2::DiffModel*"
QT_MOC_LITERAL(4, 61, 5), // "model"
QT_MOC_LITERAL(5, 67, 24), // "const Diff2::Difference*"
QT_MOC_LITERAL(6, 92, 4), // "diff"
QT_MOC_LITERAL(7, 97, 16), // "slotSetSelection"
QT_MOC_LITERAL(8, 114, 17), // "slotModelsChanged"
QT_MOC_LITERAL(9, 132, 27), // "const Diff2::DiffModelList*"
QT_MOC_LITERAL(10, 160, 9), // "modelList"
QT_MOC_LITERAL(11, 170, 15), // "slotKompareInfo"
QT_MOC_LITERAL(12, 186, 14), // "Kompare::Info*"
QT_MOC_LITERAL(13, 201, 4), // "info"
QT_MOC_LITERAL(14, 206, 30), // "slotSrcDirTreeSelectionChanged"
QT_MOC_LITERAL(15, 237, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(16, 254, 4), // "item"
QT_MOC_LITERAL(17, 259, 31), // "slotDestDirTreeSelectionChanged"
QT_MOC_LITERAL(18, 291, 28), // "slotFileListSelectionChanged"
QT_MOC_LITERAL(19, 320, 31), // "slotChangesListSelectionChanged"
QT_MOC_LITERAL(20, 352, 19), // "slotApplyDifference"
QT_MOC_LITERAL(21, 372, 5), // "apply"
QT_MOC_LITERAL(22, 378, 23), // "slotApplyAllDifferences"
QT_MOC_LITERAL(23, 402, 17) // "buildTreeInMemory"

    },
    "KompareNavTreePart\0selectionChanged\0"
    "\0const Diff2::DiffModel*\0model\0"
    "const Diff2::Difference*\0diff\0"
    "slotSetSelection\0slotModelsChanged\0"
    "const Diff2::DiffModelList*\0modelList\0"
    "slotKompareInfo\0Kompare::Info*\0info\0"
    "slotSrcDirTreeSelectionChanged\0"
    "QTreeWidgetItem*\0item\0"
    "slotDestDirTreeSelectionChanged\0"
    "slotFileListSelectionChanged\0"
    "slotChangesListSelectionChanged\0"
    "slotApplyDifference\0apply\0"
    "slotApplyAllDifferences\0buildTreeInMemory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareNavTreePart[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   84,    2, 0x06 /* Public */,
       1,    1,   89,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,   92,    2, 0x0a /* Public */,
       7,    1,   97,    2, 0x0a /* Public */,
       8,    1,  100,    2, 0x0a /* Public */,
      11,    1,  103,    2, 0x0a /* Public */,
      14,    1,  106,    2, 0x08 /* Private */,
      17,    1,  109,    2, 0x08 /* Private */,
      18,    1,  112,    2, 0x08 /* Private */,
      19,    1,  115,    2, 0x08 /* Private */,
      20,    1,  118,    2, 0x08 /* Private */,
      22,    1,  121,    2, 0x08 /* Private */,
      20,    2,  124,    2, 0x08 /* Private */,
      23,    0,  129,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 5,    6,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,    6,   21,
    QMetaType::Void,

       0        // eod
};

void KompareNavTreePart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KompareNavTreePart *_t = static_cast<KompareNavTreePart *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->selectionChanged((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1])),(*reinterpret_cast< const Diff2::Difference*(*)>(_a[2]))); break;
        case 1: _t->selectionChanged((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 2: _t->slotSetSelection((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1])),(*reinterpret_cast< const Diff2::Difference*(*)>(_a[2]))); break;
        case 3: _t->slotSetSelection((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 4: _t->slotModelsChanged((*reinterpret_cast< const Diff2::DiffModelList*(*)>(_a[1]))); break;
        case 5: _t->slotKompareInfo((*reinterpret_cast< Kompare::Info*(*)>(_a[1]))); break;
        case 6: _t->slotSrcDirTreeSelectionChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 7: _t->slotDestDirTreeSelectionChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 8: _t->slotFileListSelectionChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 9: _t->slotChangesListSelectionChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 10: _t->slotApplyDifference((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->slotApplyAllDifferences((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->slotApplyDifference((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 13: _t->buildTreeInMemory(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KompareNavTreePart::*_t)(const Diff2::DiffModel * , const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareNavTreePart::selectionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KompareNavTreePart::*_t)(const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareNavTreePart::selectionChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject KompareNavTreePart::staticMetaObject = {
    { &KParts::ReadOnlyPart::staticMetaObject, qt_meta_stringdata_KompareNavTreePart.data,
      qt_meta_data_KompareNavTreePart,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareNavTreePart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareNavTreePart::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareNavTreePart.stringdata0))
        return static_cast<void*>(const_cast< KompareNavTreePart*>(this));
    return KParts::ReadOnlyPart::qt_metacast(_clname);
}

int KompareNavTreePart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ReadOnlyPart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void KompareNavTreePart::selectionChanged(const Diff2::DiffModel * _t1, const Diff2::Difference * _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KompareNavTreePart::selectionChanged(const Diff2::Difference * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_END_MOC_NAMESPACE
