/****************************************************************************
** Meta object code from reading C++ file 'komparelistview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kompare-17.04.3/komparepart/komparelistview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'komparelistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KompareListView_t {
    QByteArrayData data[18];
    char stringdata0[222];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareListView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareListView_t qt_meta_stringdata_KompareListView = {
    {
QT_MOC_LITERAL(0, 0, 15), // "KompareListView"
QT_MOC_LITERAL(1, 16, 17), // "differenceClicked"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 24), // "const Diff2::Difference*"
QT_MOC_LITERAL(4, 60, 4), // "diff"
QT_MOC_LITERAL(5, 65, 15), // "applyDifference"
QT_MOC_LITERAL(6, 81, 5), // "apply"
QT_MOC_LITERAL(7, 87, 7), // "resized"
QT_MOC_LITERAL(8, 95, 16), // "slotSetSelection"
QT_MOC_LITERAL(9, 112, 23), // "const Diff2::DiffModel*"
QT_MOC_LITERAL(10, 136, 5), // "model"
QT_MOC_LITERAL(11, 142, 10), // "setXOffset"
QT_MOC_LITERAL(12, 153, 1), // "x"
QT_MOC_LITERAL(13, 155, 10), // "scrollToId"
QT_MOC_LITERAL(14, 166, 2), // "id"
QT_MOC_LITERAL(15, 169, 8), // "scrollId"
QT_MOC_LITERAL(16, 178, 19), // "slotApplyDifference"
QT_MOC_LITERAL(17, 198, 23) // "slotApplyAllDifferences"

    },
    "KompareListView\0differenceClicked\0\0"
    "const Diff2::Difference*\0diff\0"
    "applyDifference\0apply\0resized\0"
    "slotSetSelection\0const Diff2::DiffModel*\0"
    "model\0setXOffset\0x\0scrollToId\0id\0"
    "scrollId\0slotApplyDifference\0"
    "slotApplyAllDifferences"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareListView[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       5,    1,   72,    2, 0x06 /* Public */,
       7,    0,   75,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    2,   76,    2, 0x0a /* Public */,
       8,    1,   81,    2, 0x0a /* Public */,
      11,    1,   84,    2, 0x0a /* Public */,
      13,    1,   87,    2, 0x0a /* Public */,
      15,    0,   90,    2, 0x0a /* Public */,
      16,    1,   91,    2, 0x0a /* Public */,
      17,    1,   94,    2, 0x0a /* Public */,
      16,    2,   97,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 3,   10,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    6,

       0        // eod
};

void KompareListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KompareListView *_t = static_cast<KompareListView *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->differenceClicked((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 1: _t->applyDifference((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->resized(); break;
        case 3: _t->slotSetSelection((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1])),(*reinterpret_cast< const Diff2::Difference*(*)>(_a[2]))); break;
        case 4: _t->slotSetSelection((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 5: _t->setXOffset((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->scrollToId((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: { int _r = _t->scrollId();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = _r; }  break;
        case 8: _t->slotApplyDifference((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->slotApplyAllDifferences((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->slotApplyDifference((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KompareListView::*_t)(const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareListView::differenceClicked)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KompareListView::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareListView::applyDifference)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (KompareListView::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareListView::resized)) {
                *result = 2;
                return;
            }
        }
    }
}

const QMetaObject KompareListView::staticMetaObject = {
    { &QTreeWidget::staticMetaObject, qt_meta_stringdata_KompareListView.data,
      qt_meta_data_KompareListView,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareListView::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareListView.stringdata0))
        return static_cast<void*>(const_cast< KompareListView*>(this));
    return QTreeWidget::qt_metacast(_clname);
}

int KompareListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KompareListView::differenceClicked(const Diff2::Difference * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KompareListView::applyDifference(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KompareListView::resized()
{
    QMetaObject::activate(this, &staticMetaObject, 2, Q_NULLPTR);
}
struct qt_meta_stringdata_KompareListViewFrame_t {
    QByteArrayData data[5];
    char stringdata0[65];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareListViewFrame_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareListViewFrame_t qt_meta_stringdata_KompareListViewFrame = {
    {
QT_MOC_LITERAL(0, 0, 20), // "KompareListViewFrame"
QT_MOC_LITERAL(1, 21, 12), // "slotSetModel"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 23), // "const Diff2::DiffModel*"
QT_MOC_LITERAL(4, 59, 5) // "model"

    },
    "KompareListViewFrame\0slotSetModel\0\0"
    "const Diff2::DiffModel*\0model"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareListViewFrame[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void KompareListViewFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KompareListViewFrame *_t = static_cast<KompareListViewFrame *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->slotSetModel((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject KompareListViewFrame::staticMetaObject = {
    { &QFrame::staticMetaObject, qt_meta_stringdata_KompareListViewFrame.data,
      qt_meta_data_KompareListViewFrame,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareListViewFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareListViewFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareListViewFrame.stringdata0))
        return static_cast<void*>(const_cast< KompareListViewFrame*>(this));
    return QFrame::qt_metacast(_clname);
}

int KompareListViewFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_KompareListViewItemDelegate_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareListViewItemDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareListViewItemDelegate_t qt_meta_stringdata_KompareListViewItemDelegate = {
    {
QT_MOC_LITERAL(0, 0, 27) // "KompareListViewItemDelegate"

    },
    "KompareListViewItemDelegate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareListViewItemDelegate[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void KompareListViewItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    Q_UNUSED(_o);
    Q_UNUSED(_id);
    Q_UNUSED(_c);
    Q_UNUSED(_a);
}

const QMetaObject KompareListViewItemDelegate::staticMetaObject = {
    { &QStyledItemDelegate::staticMetaObject, qt_meta_stringdata_KompareListViewItemDelegate.data,
      qt_meta_data_KompareListViewItemDelegate,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareListViewItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareListViewItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareListViewItemDelegate.stringdata0))
        return static_cast<void*>(const_cast< KompareListViewItemDelegate*>(this));
    return QStyledItemDelegate::qt_metacast(_clname);
}

int KompareListViewItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStyledItemDelegate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    return _id;
}
QT_END_MOC_NAMESPACE
