#include <klocalizedstring.h>

/********************************************************************************
** Form generated from reading UI file 'komparesaveoptionsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.6.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KOMPARESAVEOPTIONSBASE_H
#define UI_KOMPARESAVEOPTIONSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <kurlrequester.h>

QT_BEGIN_NAMESPACE

class Ui_KompareSaveOptionsBase
{
public:
    QGridLayout *gridLayout;
    QGroupBox *GroupBox2;
    QHBoxLayout *hboxLayout;
    KUrlRequester *m_directoryRequester;
    QGroupBox *m_CommandLineGB;
    QHBoxLayout *hboxLayout1;
    QLabel *m_CommandLineLabel;
    QGroupBox *m_OptionsGB;
    QVBoxLayout *vboxLayout;
    QCheckBox *m_SmallerChangesCB;
    QCheckBox *m_LargeFilesCB;
    QCheckBox *m_IgnoreCaseCB;
    QCheckBox *m_ExpandTabsCB;
    QCheckBox *m_IgnoreEmptyLinesCB;
    QCheckBox *m_IgnoreWhiteSpaceCB;
    QCheckBox *m_FunctionNamesCB;
    QCheckBox *m_RecursiveCB;
    QCheckBox *m_NewFilesCB;
    QGroupBox *m_FormatGB;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_ContextRB;
    QRadioButton *m_EdRB;
    QRadioButton *m_NormalRB;
    QRadioButton *m_RCSRB;
    QRadioButton *m_UnifiedRB;
    QRadioButton *m_SideBySideRB;
    QHBoxLayout *hboxLayout2;
    QLabel *m_ContextLinesLabel;
    QSpinBox *m_ContextLinesSB;

    void setupUi(QWidget *KompareSaveOptionsBase)
    {
        if (KompareSaveOptionsBase->objectName().isEmpty())
            KompareSaveOptionsBase->setObjectName(QStringLiteral("KompareSaveOptionsBase"));
        KompareSaveOptionsBase->resize(558, 399);
        gridLayout = new QGridLayout(KompareSaveOptionsBase);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName(QStringLiteral("gridLayout"));
        GroupBox2 = new QGroupBox(KompareSaveOptionsBase);
        GroupBox2->setObjectName(QStringLiteral("GroupBox2"));
        QSizePolicy sizePolicy(static_cast<QSizePolicy::Policy>(1), static_cast<QSizePolicy::Policy>(1));
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(GroupBox2->sizePolicy().hasHeightForWidth());
        GroupBox2->setSizePolicy(sizePolicy);
        hboxLayout = new QHBoxLayout(GroupBox2);
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName(QStringLiteral("hboxLayout"));
        m_directoryRequester = new KUrlRequester(GroupBox2);
        m_directoryRequester->setObjectName(QStringLiteral("m_directoryRequester"));
        QSizePolicy sizePolicy1(static_cast<QSizePolicy::Policy>(7), static_cast<QSizePolicy::Policy>(5));
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_directoryRequester->sizePolicy().hasHeightForWidth());
        m_directoryRequester->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(m_directoryRequester);


        gridLayout->addWidget(GroupBox2, 1, 0, 1, 2);

        m_CommandLineGB = new QGroupBox(KompareSaveOptionsBase);
        m_CommandLineGB->setObjectName(QStringLiteral("m_CommandLineGB"));
        QSizePolicy sizePolicy2(static_cast<QSizePolicy::Policy>(3), static_cast<QSizePolicy::Policy>(3));
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_CommandLineGB->sizePolicy().hasHeightForWidth());
        m_CommandLineGB->setSizePolicy(sizePolicy2);
        m_CommandLineGB->setAlignment(Qt::AlignVCenter|Qt::AlignLeft);
        hboxLayout1 = new QHBoxLayout(m_CommandLineGB);
#ifndef UI_Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName(QStringLiteral("hboxLayout1"));
        m_CommandLineLabel = new QLabel(m_CommandLineGB);
        m_CommandLineLabel->setObjectName(QStringLiteral("m_CommandLineLabel"));
        m_CommandLineLabel->setWordWrap(false);

        hboxLayout1->addWidget(m_CommandLineLabel);


        gridLayout->addWidget(m_CommandLineGB, 2, 0, 1, 2);

        m_OptionsGB = new QGroupBox(KompareSaveOptionsBase);
        m_OptionsGB->setObjectName(QStringLiteral("m_OptionsGB"));
        vboxLayout = new QVBoxLayout(m_OptionsGB);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QStringLiteral("vboxLayout"));
        m_SmallerChangesCB = new QCheckBox(m_OptionsGB);
        m_SmallerChangesCB->setObjectName(QStringLiteral("m_SmallerChangesCB"));
        m_SmallerChangesCB->setChecked(true);

        vboxLayout->addWidget(m_SmallerChangesCB);

        m_LargeFilesCB = new QCheckBox(m_OptionsGB);
        m_LargeFilesCB->setObjectName(QStringLiteral("m_LargeFilesCB"));
        m_LargeFilesCB->setChecked(true);

        vboxLayout->addWidget(m_LargeFilesCB);

        m_IgnoreCaseCB = new QCheckBox(m_OptionsGB);
        m_IgnoreCaseCB->setObjectName(QStringLiteral("m_IgnoreCaseCB"));

        vboxLayout->addWidget(m_IgnoreCaseCB);

        m_ExpandTabsCB = new QCheckBox(m_OptionsGB);
        m_ExpandTabsCB->setObjectName(QStringLiteral("m_ExpandTabsCB"));

        vboxLayout->addWidget(m_ExpandTabsCB);

        m_IgnoreEmptyLinesCB = new QCheckBox(m_OptionsGB);
        m_IgnoreEmptyLinesCB->setObjectName(QStringLiteral("m_IgnoreEmptyLinesCB"));

        vboxLayout->addWidget(m_IgnoreEmptyLinesCB);

        m_IgnoreWhiteSpaceCB = new QCheckBox(m_OptionsGB);
        m_IgnoreWhiteSpaceCB->setObjectName(QStringLiteral("m_IgnoreWhiteSpaceCB"));

        vboxLayout->addWidget(m_IgnoreWhiteSpaceCB);

        m_FunctionNamesCB = new QCheckBox(m_OptionsGB);
        m_FunctionNamesCB->setObjectName(QStringLiteral("m_FunctionNamesCB"));
        m_FunctionNamesCB->setChecked(true);

        vboxLayout->addWidget(m_FunctionNamesCB);

        m_RecursiveCB = new QCheckBox(m_OptionsGB);
        m_RecursiveCB->setObjectName(QStringLiteral("m_RecursiveCB"));
        m_RecursiveCB->setChecked(true);

        vboxLayout->addWidget(m_RecursiveCB);

        m_NewFilesCB = new QCheckBox(m_OptionsGB);
        m_NewFilesCB->setObjectName(QStringLiteral("m_NewFilesCB"));
        m_NewFilesCB->setChecked(true);
        m_NewFilesCB->setTristate(false);

        vboxLayout->addWidget(m_NewFilesCB);


        gridLayout->addWidget(m_OptionsGB, 0, 1, 1, 1);

        m_FormatGB = new QGroupBox(KompareSaveOptionsBase);
        m_FormatGB->setObjectName(QStringLiteral("m_FormatGB"));
        vboxLayout1 = new QVBoxLayout(m_FormatGB);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName(QStringLiteral("vboxLayout1"));
        m_ContextRB = new QRadioButton(m_FormatGB);
        m_ContextRB->setObjectName(QStringLiteral("m_ContextRB"));

        vboxLayout1->addWidget(m_ContextRB);

        m_EdRB = new QRadioButton(m_FormatGB);
        m_EdRB->setObjectName(QStringLiteral("m_EdRB"));

        vboxLayout1->addWidget(m_EdRB);

        m_NormalRB = new QRadioButton(m_FormatGB);
        m_NormalRB->setObjectName(QStringLiteral("m_NormalRB"));

        vboxLayout1->addWidget(m_NormalRB);

        m_RCSRB = new QRadioButton(m_FormatGB);
        m_RCSRB->setObjectName(QStringLiteral("m_RCSRB"));

        vboxLayout1->addWidget(m_RCSRB);

        m_UnifiedRB = new QRadioButton(m_FormatGB);
        m_UnifiedRB->setObjectName(QStringLiteral("m_UnifiedRB"));
        m_UnifiedRB->setChecked(true);

        vboxLayout1->addWidget(m_UnifiedRB);

        m_SideBySideRB = new QRadioButton(m_FormatGB);
        m_SideBySideRB->setObjectName(QStringLiteral("m_SideBySideRB"));
        m_SideBySideRB->setChecked(false);

        vboxLayout1->addWidget(m_SideBySideRB);

        hboxLayout2 = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QStringLiteral("hboxLayout2"));
        m_ContextLinesLabel = new QLabel(m_FormatGB);
        m_ContextLinesLabel->setObjectName(QStringLiteral("m_ContextLinesLabel"));
        m_ContextLinesLabel->setWordWrap(false);

        hboxLayout2->addWidget(m_ContextLinesLabel);

        m_ContextLinesSB = new QSpinBox(m_FormatGB);
        m_ContextLinesSB->setObjectName(QStringLiteral("m_ContextLinesSB"));
        m_ContextLinesSB->setEnabled(true);
        m_ContextLinesSB->setMaximum(65535);
        m_ContextLinesSB->setValue(3);

        hboxLayout2->addWidget(m_ContextLinesSB);


        vboxLayout1->addLayout(hboxLayout2);


        gridLayout->addWidget(m_FormatGB, 0, 0, 1, 1);


        retranslateUi(KompareSaveOptionsBase);

        QMetaObject::connectSlotsByName(KompareSaveOptionsBase);
    } // setupUi

    void retranslateUi(QWidget *KompareSaveOptionsBase)
    {
        GroupBox2->setTitle(tr2i18n("Run Diff In", 0));
        m_CommandLineGB->setTitle(tr2i18n("Command Line", 0));
        m_CommandLineLabel->setText(tr2i18n("cd dir && diff -udHprNa -- source destination", 0));
        m_OptionsGB->setTitle(tr2i18n("Options", 0));
        m_SmallerChangesCB->setText(tr2i18n("Look for smaller changes", 0));
        m_LargeFilesCB->setText(tr2i18n("Optimize for large files", 0));
        m_IgnoreCaseCB->setText(tr2i18n("Ignore changes in case", 0));
        m_ExpandTabsCB->setText(tr2i18n("Expand tabs to spaces", 0));
        m_IgnoreEmptyLinesCB->setText(tr2i18n("Ignore added or removed empty lines", 0));
        m_IgnoreWhiteSpaceCB->setText(tr2i18n("Ignore changes in whitespace", 0));
        m_FunctionNamesCB->setText(tr2i18n("Show function names", 0));
        m_RecursiveCB->setText(tr2i18n("Compare folders recursively", 0));
        m_NewFilesCB->setText(tr2i18n("Treat new files as empty", 0));
        m_FormatGB->setTitle(tr2i18n("Format", 0));
        m_ContextRB->setText(tr2i18n("Context", 0));
        m_EdRB->setText(tr2i18n("Ed", 0));
        m_NormalRB->setText(tr2i18n("Normal", 0));
        m_RCSRB->setText(tr2i18n("RCS", 0));
        m_UnifiedRB->setText(tr2i18n("Unified", 0));
        m_SideBySideRB->setText(tr2i18n("Side-by-side", 0));
        m_ContextLinesLabel->setText(tr2i18n("Number of context lines:", 0));
        Q_UNUSED(KompareSaveOptionsBase);
    } // retranslateUi

};

namespace Ui {
    class KompareSaveOptionsBase: public Ui_KompareSaveOptionsBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KOMPARESAVEOPTIONSBASE_H

