/****************************************************************************
** Meta object code from reading C++ file 'kompare_part.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kompare-17.04.3/komparepart/kompare_part.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kompare_part.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KomparePart_t {
    QByteArrayData data[48];
    char stringdata0[660];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KomparePart_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KomparePart_t qt_meta_stringdata_KomparePart = {
    {
QT_MOC_LITERAL(0, 0, 11), // "KomparePart"
QT_MOC_LITERAL(1, 12, 13), // "modelsChanged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 27), // "const Diff2::DiffModelList*"
QT_MOC_LITERAL(4, 55, 6), // "models"
QT_MOC_LITERAL(5, 62, 12), // "setSelection"
QT_MOC_LITERAL(6, 75, 23), // "const Diff2::DiffModel*"
QT_MOC_LITERAL(7, 99, 5), // "model"
QT_MOC_LITERAL(8, 105, 24), // "const Diff2::Difference*"
QT_MOC_LITERAL(9, 130, 4), // "diff"
QT_MOC_LITERAL(10, 135, 16), // "selectionChanged"
QT_MOC_LITERAL(11, 152, 15), // "applyDifference"
QT_MOC_LITERAL(12, 168, 5), // "apply"
QT_MOC_LITERAL(13, 174, 19), // "applyAllDifferences"
QT_MOC_LITERAL(14, 194, 13), // "configChanged"
QT_MOC_LITERAL(15, 208, 17), // "differenceClicked"
QT_MOC_LITERAL(16, 226, 10), // "lineNumber"
QT_MOC_LITERAL(17, 237, 14), // "appliedChanged"
QT_MOC_LITERAL(18, 252, 14), // "diffURLChanged"
QT_MOC_LITERAL(19, 267, 11), // "kompareInfo"
QT_MOC_LITERAL(20, 279, 14), // "Kompare::Info*"
QT_MOC_LITERAL(21, 294, 4), // "info"
QT_MOC_LITERAL(22, 299, 21), // "setStatusBarModelInfo"
QT_MOC_LITERAL(23, 321, 10), // "modelIndex"
QT_MOC_LITERAL(24, 332, 15), // "differenceIndex"
QT_MOC_LITERAL(25, 348, 10), // "modelCount"
QT_MOC_LITERAL(26, 359, 15), // "differenceCount"
QT_MOC_LITERAL(27, 375, 12), // "appliedCount"
QT_MOC_LITERAL(28, 388, 10), // "diffString"
QT_MOC_LITERAL(29, 399, 7), // "saveAll"
QT_MOC_LITERAL(30, 407, 8), // "saveDiff"
QT_MOC_LITERAL(31, 416, 13), // "slotFilePrint"
QT_MOC_LITERAL(32, 430, 20), // "slotFilePrintPreview"
QT_MOC_LITERAL(33, 451, 13), // "slotSetStatus"
QT_MOC_LITERAL(34, 465, 15), // "Kompare::Status"
QT_MOC_LITERAL(35, 481, 6), // "status"
QT_MOC_LITERAL(36, 488, 13), // "slotShowError"
QT_MOC_LITERAL(37, 502, 5), // "error"
QT_MOC_LITERAL(38, 508, 8), // "slotSwap"
QT_MOC_LITERAL(39, 517, 17), // "slotShowDiffstats"
QT_MOC_LITERAL(40, 535, 15), // "slotRefreshDiff"
QT_MOC_LITERAL(41, 551, 18), // "optionsPreferences"
QT_MOC_LITERAL(42, 570, 13), // "updateActions"
QT_MOC_LITERAL(43, 584, 13), // "updateCaption"
QT_MOC_LITERAL(44, 598, 12), // "updateStatus"
QT_MOC_LITERAL(45, 611, 19), // "compareAndUpdateAll"
QT_MOC_LITERAL(46, 631, 18), // "slotPaintRequested"
QT_MOC_LITERAL(47, 650, 9) // "QPrinter*"

    },
    "KomparePart\0modelsChanged\0\0"
    "const Diff2::DiffModelList*\0models\0"
    "setSelection\0const Diff2::DiffModel*\0"
    "model\0const Diff2::Difference*\0diff\0"
    "selectionChanged\0applyDifference\0apply\0"
    "applyAllDifferences\0configChanged\0"
    "differenceClicked\0lineNumber\0"
    "appliedChanged\0diffURLChanged\0kompareInfo\0"
    "Kompare::Info*\0info\0setStatusBarModelInfo\0"
    "modelIndex\0differenceIndex\0modelCount\0"
    "differenceCount\0appliedCount\0diffString\0"
    "saveAll\0saveDiff\0slotFilePrint\0"
    "slotFilePrintPreview\0slotSetStatus\0"
    "Kompare::Status\0status\0slotShowError\0"
    "error\0slotSwap\0slotShowDiffstats\0"
    "slotRefreshDiff\0optionsPreferences\0"
    "updateActions\0updateCaption\0updateStatus\0"
    "compareAndUpdateAll\0slotPaintRequested\0"
    "QPrinter*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KomparePart[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      30,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      15,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  164,    2, 0x06 /* Public */,
       5,    2,  167,    2, 0x06 /* Public */,
       5,    1,  172,    2, 0x06 /* Public */,
      10,    2,  175,    2, 0x06 /* Public */,
      10,    1,  180,    2, 0x06 /* Public */,
      11,    1,  183,    2, 0x06 /* Public */,
      13,    1,  186,    2, 0x06 /* Public */,
      11,    2,  189,    2, 0x06 /* Public */,
      14,    0,  194,    2, 0x06 /* Public */,
      15,    1,  195,    2, 0x06 /* Public */,
      17,    0,  198,    2, 0x06 /* Public */,
      18,    0,  199,    2, 0x06 /* Public */,
      19,    1,  200,    2, 0x06 /* Public */,
      22,    5,  203,    2, 0x06 /* Public */,
      28,    1,  214,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      29,    0,  217,    2, 0x0a /* Public */,
      30,    0,  218,    2, 0x0a /* Public */,
      31,    0,  219,    2, 0x0a /* Public */,
      32,    0,  220,    2, 0x0a /* Public */,
      33,    1,  221,    2, 0x09 /* Protected */,
      36,    1,  224,    2, 0x09 /* Protected */,
      38,    0,  227,    2, 0x09 /* Protected */,
      39,    0,  228,    2, 0x09 /* Protected */,
      40,    0,  229,    2, 0x09 /* Protected */,
      41,    0,  230,    2, 0x09 /* Protected */,
      42,    0,  231,    2, 0x09 /* Protected */,
      43,    0,  232,    2, 0x09 /* Protected */,
      44,    0,  233,    2, 0x09 /* Protected */,
      45,    0,  234,    2, 0x09 /* Protected */,
      46,    1,  235,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8,    7,    9,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8,    7,    9,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, 0x80000000 | 8, QMetaType::Bool,    2,   12,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 20,   21,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,   23,   24,   25,   26,   27,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 34,   35,
    QMetaType::Void, QMetaType::QString,   37,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 47,    2,

       0        // eod
};

void KomparePart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KomparePart *_t = static_cast<KomparePart *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->modelsChanged((*reinterpret_cast< const Diff2::DiffModelList*(*)>(_a[1]))); break;
        case 1: _t->setSelection((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1])),(*reinterpret_cast< const Diff2::Difference*(*)>(_a[2]))); break;
        case 2: _t->setSelection((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 3: _t->selectionChanged((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1])),(*reinterpret_cast< const Diff2::Difference*(*)>(_a[2]))); break;
        case 4: _t->selectionChanged((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 5: _t->applyDifference((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->applyAllDifferences((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->applyDifference((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 8: _t->configChanged(); break;
        case 9: _t->differenceClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->appliedChanged(); break;
        case 11: _t->diffURLChanged(); break;
        case 12: _t->kompareInfo((*reinterpret_cast< Kompare::Info*(*)>(_a[1]))); break;
        case 13: _t->setStatusBarModelInfo((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 14: _t->diffString((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: { bool _r = _t->saveAll();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 16: _t->saveDiff(); break;
        case 17: _t->slotFilePrint(); break;
        case 18: _t->slotFilePrintPreview(); break;
        case 19: _t->slotSetStatus((*reinterpret_cast< Kompare::Status(*)>(_a[1]))); break;
        case 20: _t->slotShowError((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 21: _t->slotSwap(); break;
        case 22: _t->slotShowDiffstats(); break;
        case 23: _t->slotRefreshDiff(); break;
        case 24: _t->optionsPreferences(); break;
        case 25: _t->updateActions(); break;
        case 26: _t->updateCaption(); break;
        case 27: _t->updateStatus(); break;
        case 28: _t->compareAndUpdateAll(); break;
        case 29: _t->slotPaintRequested((*reinterpret_cast< QPrinter*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KomparePart::*_t)(const Diff2::DiffModelList * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::modelsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(const Diff2::DiffModel * , const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::setSelection)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::setSelection)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(const Diff2::DiffModel * , const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::selectionChanged)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::selectionChanged)) {
                *result = 4;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::applyDifference)) {
                *result = 5;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::applyAllDifferences)) {
                *result = 6;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(const Diff2::Difference * , bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::applyDifference)) {
                *result = 7;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::configChanged)) {
                *result = 8;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::differenceClicked)) {
                *result = 9;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::appliedChanged)) {
                *result = 10;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::diffURLChanged)) {
                *result = 11;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(Kompare::Info * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::kompareInfo)) {
                *result = 12;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(int , int , int , int , int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::setStatusBarModelInfo)) {
                *result = 13;
                return;
            }
        }
        {
            typedef void (KomparePart::*_t)(const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KomparePart::diffString)) {
                *result = 14;
                return;
            }
        }
    }
}

const QMetaObject KomparePart::staticMetaObject = {
    { &KParts::ReadWritePart::staticMetaObject, qt_meta_stringdata_KomparePart.data,
      qt_meta_data_KomparePart,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KomparePart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KomparePart::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KomparePart.stringdata0))
        return static_cast<void*>(const_cast< KomparePart*>(this));
    if (!strcmp(_clname, "KompareInterface"))
        return static_cast< KompareInterface*>(const_cast< KomparePart*>(this));
    if (!strcmp(_clname, "com.kde.Kompare.KompareInterface/4.0"))
        return static_cast< KompareInterface*>(const_cast< KomparePart*>(this));
    return KParts::ReadWritePart::qt_metacast(_clname);
}

int KomparePart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ReadWritePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 30;
    }
    return _id;
}

// SIGNAL 0
void KomparePart::modelsChanged(const Diff2::DiffModelList * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KomparePart::setSelection(const Diff2::DiffModel * _t1, const Diff2::Difference * _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KomparePart::setSelection(const Diff2::Difference * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KomparePart::selectionChanged(const Diff2::DiffModel * _t1, const Diff2::Difference * _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KomparePart::selectionChanged(const Diff2::Difference * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void KomparePart::applyDifference(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void KomparePart::applyAllDifferences(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void KomparePart::applyDifference(const Diff2::Difference * _t1, bool _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void KomparePart::configChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, Q_NULLPTR);
}

// SIGNAL 9
void KomparePart::differenceClicked(int _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void KomparePart::appliedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, Q_NULLPTR);
}

// SIGNAL 11
void KomparePart::diffURLChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, Q_NULLPTR);
}

// SIGNAL 12
void KomparePart::kompareInfo(Kompare::Info * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void KomparePart::setStatusBarModelInfo(int _t1, int _t2, int _t3, int _t4, int _t5)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)), const_cast<void*>(reinterpret_cast<const void*>(&_t5)) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void KomparePart::diffString(const QString & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}
QT_END_MOC_NAMESPACE
