/****************************************************************************
** Meta object code from reading C++ file 'komparesplitter.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kompare-17.04.3/komparepart/komparesplitter.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'komparesplitter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KompareSplitter_t {
    QByteArrayData data[29];
    char stringdata0[449];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareSplitter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareSplitter_t qt_meta_stringdata_KompareSplitter = {
    {
QT_MOC_LITERAL(0, 0, 15), // "KompareSplitter"
QT_MOC_LITERAL(1, 16, 13), // "configChanged"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 15), // "scrollViewsToId"
QT_MOC_LITERAL(4, 47, 2), // "id"
QT_MOC_LITERAL(5, 50, 10), // "setXOffset"
QT_MOC_LITERAL(6, 61, 1), // "x"
QT_MOC_LITERAL(7, 63, 16), // "selectionChanged"
QT_MOC_LITERAL(8, 80, 24), // "const Diff2::Difference*"
QT_MOC_LITERAL(9, 105, 4), // "diff"
QT_MOC_LITERAL(10, 110, 14), // "slotScrollToId"
QT_MOC_LITERAL(11, 125, 27), // "slotDelayedUpdateScrollBars"
QT_MOC_LITERAL(12, 153, 20), // "slotUpdateScrollBars"
QT_MOC_LITERAL(13, 174, 29), // "slotDelayedUpdateVScrollValue"
QT_MOC_LITERAL(14, 204, 22), // "slotUpdateVScrollValue"
QT_MOC_LITERAL(15, 227, 13), // "keyPressEvent"
QT_MOC_LITERAL(16, 241, 10), // "QKeyEvent*"
QT_MOC_LITERAL(17, 252, 1), // "e"
QT_MOC_LITERAL(18, 254, 19), // "slotApplyDifference"
QT_MOC_LITERAL(19, 274, 5), // "apply"
QT_MOC_LITERAL(20, 280, 23), // "slotApplyAllDifferences"
QT_MOC_LITERAL(21, 304, 16), // "slotSetSelection"
QT_MOC_LITERAL(22, 321, 23), // "const Diff2::DiffModel*"
QT_MOC_LITERAL(23, 345, 5), // "model"
QT_MOC_LITERAL(24, 351, 21), // "slotDifferenceClicked"
QT_MOC_LITERAL(25, 373, 17), // "slotConfigChanged"
QT_MOC_LITERAL(26, 391, 25), // "slotDelayedRepaintHandles"
QT_MOC_LITERAL(27, 417, 18), // "slotRepaintHandles"
QT_MOC_LITERAL(28, 436, 12) // "timerTimeout"

    },
    "KompareSplitter\0configChanged\0\0"
    "scrollViewsToId\0id\0setXOffset\0x\0"
    "selectionChanged\0const Diff2::Difference*\0"
    "diff\0slotScrollToId\0slotDelayedUpdateScrollBars\0"
    "slotUpdateScrollBars\0slotDelayedUpdateVScrollValue\0"
    "slotUpdateVScrollValue\0keyPressEvent\0"
    "QKeyEvent*\0e\0slotApplyDifference\0apply\0"
    "slotApplyAllDifferences\0slotSetSelection\0"
    "const Diff2::DiffModel*\0model\0"
    "slotDifferenceClicked\0slotConfigChanged\0"
    "slotDelayedRepaintHandles\0slotRepaintHandles\0"
    "timerTimeout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareSplitter[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  114,    2, 0x06 /* Public */,
       3,    1,  115,    2, 0x06 /* Public */,
       5,    1,  118,    2, 0x06 /* Public */,
       7,    1,  121,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,  124,    2, 0x0a /* Public */,
      11,    0,  127,    2, 0x0a /* Public */,
      12,    0,  128,    2, 0x0a /* Public */,
      13,    0,  129,    2, 0x0a /* Public */,
      14,    0,  130,    2, 0x0a /* Public */,
      15,    1,  131,    2, 0x0a /* Public */,
      18,    1,  134,    2, 0x0a /* Public */,
      20,    1,  137,    2, 0x0a /* Public */,
      18,    2,  140,    2, 0x0a /* Public */,
      21,    2,  145,    2, 0x0a /* Public */,
      21,    1,  150,    2, 0x0a /* Public */,
      24,    1,  153,    2, 0x0a /* Public */,
      25,    0,  156,    2, 0x0a /* Public */,
      26,    0,  157,    2, 0x09 /* Protected */,
      27,    0,  158,    2, 0x09 /* Protected */,
      28,    0,  159,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, 0x80000000 | 8,    9,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, 0x80000000 | 8, QMetaType::Bool,    9,   19,
    QMetaType::Void, 0x80000000 | 22, 0x80000000 | 8,   23,    9,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void KompareSplitter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KompareSplitter *_t = static_cast<KompareSplitter *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->configChanged(); break;
        case 1: _t->scrollViewsToId((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->setXOffset((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->selectionChanged((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 4: _t->slotScrollToId((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->slotDelayedUpdateScrollBars(); break;
        case 6: _t->slotUpdateScrollBars(); break;
        case 7: _t->slotDelayedUpdateVScrollValue(); break;
        case 8: _t->slotUpdateVScrollValue(); break;
        case 9: _t->keyPressEvent((*reinterpret_cast< QKeyEvent*(*)>(_a[1]))); break;
        case 10: _t->slotApplyDifference((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->slotApplyAllDifferences((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->slotApplyDifference((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 13: _t->slotSetSelection((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1])),(*reinterpret_cast< const Diff2::Difference*(*)>(_a[2]))); break;
        case 14: _t->slotSetSelection((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 15: _t->slotDifferenceClicked((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 16: _t->slotConfigChanged(); break;
        case 17: _t->slotDelayedRepaintHandles(); break;
        case 18: _t->slotRepaintHandles(); break;
        case 19: _t->timerTimeout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KompareSplitter::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareSplitter::configChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KompareSplitter::*_t)(int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareSplitter::scrollViewsToId)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (KompareSplitter::*_t)(int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareSplitter::setXOffset)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (KompareSplitter::*_t)(const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareSplitter::selectionChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

const QMetaObject KompareSplitter::staticMetaObject = {
    { &QSplitter::staticMetaObject, qt_meta_stringdata_KompareSplitter.data,
      qt_meta_data_KompareSplitter,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareSplitter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareSplitter::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareSplitter.stringdata0))
        return static_cast<void*>(const_cast< KompareSplitter*>(this));
    return QSplitter::qt_metacast(_clname);
}

int KompareSplitter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void KompareSplitter::configChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, Q_NULLPTR);
}

// SIGNAL 1
void KompareSplitter::scrollViewsToId(int _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KompareSplitter::setXOffset(int _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KompareSplitter::selectionChanged(const Diff2::Difference * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_END_MOC_NAMESPACE
