/****************************************************************************
** Meta object code from reading C++ file 'kompareconnectwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kompare-17.04.3/komparepart/kompareconnectwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kompareconnectwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KompareConnectWidget_t {
    QByteArrayData data[9];
    char stringdata0[135];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareConnectWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareConnectWidget_t qt_meta_stringdata_KompareConnectWidget = {
    {
QT_MOC_LITERAL(0, 0, 20), // "KompareConnectWidget"
QT_MOC_LITERAL(1, 21, 16), // "selectionChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 24), // "const Diff2::Difference*"
QT_MOC_LITERAL(4, 64, 4), // "diff"
QT_MOC_LITERAL(5, 69, 16), // "slotSetSelection"
QT_MOC_LITERAL(6, 86, 23), // "const Diff2::DiffModel*"
QT_MOC_LITERAL(7, 110, 5), // "model"
QT_MOC_LITERAL(8, 116, 18) // "slotDelayedRepaint"

    },
    "KompareConnectWidget\0selectionChanged\0"
    "\0const Diff2::Difference*\0diff\0"
    "slotSetSelection\0const Diff2::DiffModel*\0"
    "model\0slotDelayedRepaint"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareConnectWidget[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    2,   37,    2, 0x0a /* Public */,
       5,    1,   42,    2, 0x0a /* Public */,
       8,    0,   45,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 3,    7,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,

       0        // eod
};

void KompareConnectWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KompareConnectWidget *_t = static_cast<KompareConnectWidget *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->selectionChanged((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 1: _t->slotSetSelection((*reinterpret_cast< const Diff2::DiffModel*(*)>(_a[1])),(*reinterpret_cast< const Diff2::Difference*(*)>(_a[2]))); break;
        case 2: _t->slotSetSelection((*reinterpret_cast< const Diff2::Difference*(*)>(_a[1]))); break;
        case 3: _t->slotDelayedRepaint(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KompareConnectWidget::*_t)(const Diff2::Difference * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KompareConnectWidget::selectionChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject KompareConnectWidget::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_KompareConnectWidget.data,
      qt_meta_data_KompareConnectWidget,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareConnectWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareConnectWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareConnectWidget.stringdata0))
        return static_cast<void*>(const_cast< KompareConnectWidget*>(this));
    return QWidget::qt_metacast(_clname);
}

int KompareConnectWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KompareConnectWidget::selectionChanged(const Diff2::Difference * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_KompareConnectWidgetFrame_t {
    QByteArrayData data[1];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KompareConnectWidgetFrame_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KompareConnectWidgetFrame_t qt_meta_stringdata_KompareConnectWidgetFrame = {
    {
QT_MOC_LITERAL(0, 0, 25) // "KompareConnectWidgetFrame"

    },
    "KompareConnectWidgetFrame"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KompareConnectWidgetFrame[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void KompareConnectWidgetFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    Q_UNUSED(_o);
    Q_UNUSED(_id);
    Q_UNUSED(_c);
    Q_UNUSED(_a);
}

const QMetaObject KompareConnectWidgetFrame::staticMetaObject = {
    { &QSplitterHandle::staticMetaObject, qt_meta_stringdata_KompareConnectWidgetFrame.data,
      qt_meta_data_KompareConnectWidgetFrame,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KompareConnectWidgetFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KompareConnectWidgetFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KompareConnectWidgetFrame.stringdata0))
        return static_cast<void*>(const_cast< KompareConnectWidgetFrame*>(this));
    return QSplitterHandle::qt_metacast(_clname);
}

int KompareConnectWidgetFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitterHandle::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    return _id;
}
QT_END_MOC_NAMESPACE
